@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -12,20 +12,13 @@
  * standalone mini error implementation
  */
 
-#ifdef _WIN32
-#include <config.h>
-
-#ifdef GVDLL
-#define _BLD_sfio 1
-#endif
-#endif
-
 #include <ast/ast.h>
 #include <ast/error.h>
-#include <cgraph/exit.h>
 #include <stddef.h>
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
+#include <util/exit.h>
 
 Error_info_t error_info;
 
@@ -36,7 +29,7 @@ void setErrorFileLine (char* src, int line) {
 }
 void setErrorId (char* id) { error_info.id = id; }
 void setErrorErrors (int errors) { error_info.errors = errors; }
-int  getErrorErrors () { return error_info.errors; }
+int  getErrorErrors (void) { return error_info.errors; }
 void setTraceLevel (int i) { error_info.trace = i; }
 
 void errorv(const char *id, int level, const char *s, va_list ap)
@@ -53,36 +46,36 @@ void errorv(const char *id, int level, const char *s, va_list ap)
     const char *prefix;
     if (level && ((prefix = error_info.id) || (prefix = id))) {
 	if (flags & ERROR_USAGE)
-	    sfprintf(sfstderr, "Usage: %s ", prefix);
+	    fprintf(stderr, "Usage: %s ", prefix);
 	else
-	    sfprintf(sfstderr, "%s: ", prefix);
+	    fprintf(stderr, "%s: ", prefix);
     }
     if (flags & ERROR_USAGE)
 	/*nop */ ;
     else if (level < 0) {
 	int i;
 	for (i = 0; i < error_info.indent; i++)
-	    sfprintf(sfstderr, "  ");
-	sfprintf(sfstderr, "debug%d: ", level);
+	    fprintf(stderr, "  ");
+	fprintf(stderr, "debug%d: ", level);
     } else if (level) {
 	if (level == ERROR_WARNING) {
-	    sfprintf(sfstderr, "warning: ");
+	    fprintf(stderr, "warning: ");
 	    error_info.warnings++;
 	} else {
 	    error_info.errors++;
 	    if (level == ERROR_PANIC)
-		sfprintf(sfstderr, "panic: ");
+		fprintf(stderr, "panic: ");
 	}
 	if (error_info.line) {
 	    if (error_info.file && *error_info.file)
-		sfprintf(sfstderr, "\"%s\", ", error_info.file);
-	    sfprintf(sfstderr, "line %d: ", error_info.line);
+		fprintf(stderr, "\"%s\", ", error_info.file);
+	    fprintf(stderr, "line %d: ", error_info.line);
 	}
     }
-    sfvprintf(sfstderr, s, ap);
+    vfprintf(stderr, s, ap);
     if (flags & ERROR_SYSTEM)
-	sfprintf(sfstderr, "\n%s", strerror(errno));
-    sfprintf(sfstderr, "\n");
+	fprintf(stderr, "\n%s", strerror(errno));
+    fprintf(stderr, "\n");
     if (level >= ERROR_FATAL)
 	graphviz_exit(level - ERROR_FATAL + 1);
 }
