@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,13 +17,13 @@
 #include <math.h>
 #include <cgraph/cgraph.h>     /* for agerr() and friends */
 #include <neatogen/delaunay.h>
-#include <common/memory.h>
+#include <util/alloc.h>
+#include <util/sort.h>
 
 #ifdef HAVE_GTS
 #include <gts.h>
 
-static gint triangle_is_hole(void *triangle, void *ignored)
-{
+static int triangle_is_hole(void *triangle, void *ignored) {
     GtsTriangle *t = triangle;
     (void)ignored;
 
@@ -40,8 +40,7 @@ static gint triangle_is_hole(void *triangle, void *ignored)
     return FALSE;
 }
 
-static guint delaunay_remove_holes(GtsSurface * surface)
-{
+static unsigned delaunay_remove_holes(GtsSurface *surface) {
     return gts_surface_foreach_face_remove(surface, triangle_is_hole, NULL);
 }
 
@@ -144,8 +143,8 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
 {
     int i;
     GtsSurface *surface;
-    GVertex **vertices = N_GNEW(npt, GVertex *);
-    GtsEdge **edges = N_GNEW(nsegs, GtsEdge*);
+    GVertex **vertices = gv_calloc(npt, sizeof(GVertex *));
+    GtsEdge **edges = gv_calloc(nsegs, sizeof(GtsEdge *));
     GSList *list = NULL;
     GtsVertex *v1, *v2, *v3;
     GtsTriangle *t;
@@ -198,7 +197,7 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
 	/* if v != NULL, it is a previously added pt with the same
 	 * coordinates as v4, in which case we replace v4 with v
 	 */
-	if (v) {
+	if (v && v4 != v) {
 	    gts_vertex_replace(v4, v);
 	}
     }
@@ -229,7 +228,7 @@ typedef struct {
     v_data *delaunay;
 } estats;
     
-static gint cnt_edge(void *edge, void *stats) {
+static int cnt_edge(void *edge, void *stats) {
     GtsSegment *e = edge;
     estats *sp = stats;
 
@@ -248,7 +247,7 @@ edgeStats (GtsSurface* s, estats* sp)
     gts_surface_foreach_edge(s, cnt_edge, sp);
 }
 
-static gint add_edge(void *edge, void *data) {
+static int add_edge(void *edge, void *data) {
     GtsSegment *e = edge;
     v_data *delaunay = data;
 
@@ -262,15 +261,13 @@ static gint add_edge(void *edge, void *data) {
 }
 
 static v_data *delaunay_triangulation(double *x, double *y, int n) {
-    v_data *delaunay;
     GtsSurface* s = tri(x, y, n, NULL, 0, 1);
     int i, nedges;
-    int* edges;
     estats stats;
 
     if (!s) return NULL;
 
-    delaunay = N_GNEW(n, v_data);
+    v_data *delaunay = gv_calloc(n, sizeof(v_data));
 
     for (i = 0; i < n; i++) {
 	delaunay[i].ewgts = NULL;
@@ -281,7 +278,7 @@ static v_data *delaunay_triangulation(double *x, double *y, int n) {
     stats.delaunay = delaunay;
     edgeStats (s, &stats);
     nedges = stats.n;
-    edges = N_GNEW(2 * nedges + n, int);
+    int *edges = gv_calloc(2 * nedges + n, sizeof(int));
 
     for (i = 0; i < n; i++) {
 	delaunay[i].edges = edges;
@@ -301,7 +298,7 @@ typedef struct {
     int* edges;
 } estate;
 
-static gint addEdge(void *edge, void *state) {
+static int addEdge(void *edge, void *state) {
     GtsSegment *e = edge;
     estate *es = state;
 
@@ -315,14 +312,10 @@ static gint addEdge(void *edge, void *state) {
     return 0;
 }
 
-// when moving to C11, qsort_s should be used instead of having a global
-// variable
-static double* _vals;
-typedef int (*qsort_cmpf) (const void *, const void *);
-
-static int 
-vcmp (int* a, int* b)
-{
+static int vcmp(const void *x, const void *y, void *values) {
+    const int *a = x;
+    const int *b = y;
+    const double *_vals = values;
     double va = _vals[*a];
     double vb = _vals[*b];
 
@@ -359,27 +352,23 @@ int *delaunay_tri(double *x, double *y, int n, int* pnedges)
     *pnedges = nedges = stats.n;
 
     if (nedges) {
-	edges = N_GNEW(2 * nedges, int);
+	edges = gv_calloc(2 * nedges, sizeof(int));
 	state.n = 0;
 	state.edges = edges;
 	gts_surface_foreach_edge(s, addEdge, &state);
     }
     else {
-	int* vs = N_GNEW(n, int);
+	int* vs = gv_calloc(n, sizeof(int));
 	int* ip;
 	int i, hd, tl;
 
 	*pnedges = nedges = n-1;
-	ip = edges = N_GNEW(2 * nedges, int);
+	ip = edges = gv_calloc(2 * nedges, sizeof(int));
 
 	for (i = 0; i < n; i++)
 	    vs[i] = i;
 
-	if (x[0] == x[1])  /* vertical line */ 
-	    _vals = y;
-	else              
-	    _vals = x;
-	qsort (vs, n, sizeof(int), (qsort_cmpf)vcmp);
+	gv_sort(vs, n, sizeof(int), vcmp, x[0] == x[1] /* vertical line? */ ? y : x);
 
 	tl = vs[0];
 	for (i = 1; i < n; i++) {
@@ -397,7 +386,7 @@ int *delaunay_tri(double *x, double *y, int n, int* pnedges)
     return edges;
 }
 
-static gint cntFace(void *face, void *data) {
+static int cntFace(void *face, void *data) {
     GFace *fp = face;
     int *ip = data;
 
@@ -418,7 +407,7 @@ typedef struct {
     int* neigh;
 } ninfo;
 
-static gint addNeighbor(void *face, void *ni) {
+static int addNeighbor(void *face, void *ni) {
     GFace *f = face;
     ninfo *es = ni;
 
@@ -428,7 +417,7 @@ static gint addNeighbor(void *face, void *ni) {
     return 0;
 }
 
-static gint addFace(void *face, void *state) {
+static int addFace(void *face, void *state) {
     GFace *f = face;
     fstate *es = state;
 
@@ -452,7 +441,7 @@ static gint addFace(void *face, void *state) {
     return 0;
 }
 
-static gint addTri(void *face, void *state) {
+static int addTri(void *face, void *state) {
     GFace *f = face;
     fstate *es = state;
 
@@ -482,19 +471,16 @@ mkSurface (double *x, double *y, int n, int* segs, int nsegs)
     estats stats;
     estate state;
     fstate statf;
-    surface_t* sf;
     int nfaces = 0;
-    int* faces; 
-    int* neigh; 
 
     if (!s) return NULL;
 
-    sf = GNEW(surface_t);
+    surface_t *sf = gv_alloc(sizeof(surface_t));
     stats.n = 0;
     stats.delaunay = NULL;
     edgeStats (s, &stats);
     nsegs = stats.n;
-    segs = N_GNEW(2 * nsegs, int);
+    segs = gv_calloc(2 * nsegs, sizeof(int));
 
     state.n = 0;
     state.edges = segs;
@@ -502,8 +488,8 @@ mkSurface (double *x, double *y, int n, int* segs, int nsegs)
 
     gts_surface_foreach_face(s, cntFace, &nfaces);
 
-    faces = N_GNEW(3 * nfaces, int);
-    neigh = N_GNEW(3 * nfaces, int);
+    int *faces = gv_calloc(3 * nfaces, sizeof(int));
+    int *neigh = gv_calloc(3 * nfaces, sizeof(int));
 
     statf.faces = faces;
     statf.neigh = neigh;
@@ -540,7 +526,7 @@ get_triangles (double *x, int n, int* tris)
     if (!s) return NULL;
 
     gts_surface_foreach_face(s, cntFace, &nfaces);
-    statf.faces = N_GNEW(3 * nfaces, int);
+    statf.faces = gv_calloc(3 * nfaces, sizeof(int));
     gts_surface_foreach_face(s, addTri, &statf);
 
     gts_object_destroy (GTS_OBJECT (s));
@@ -572,7 +558,7 @@ get_triangles (double *x, int n, int* tris)
 
     in.numberofpoints = n;
     in.numberofpointattributes = 0;
-    in.pointlist = N_GNEW(in.numberofpoints * 2, REAL);
+    in.pointlist = gv_calloc(in.numberofpoints * 2, sizeof(REAL));
 
     for (i = 0; i < n; i++){
 	in.pointlist[i*2] = x[i*2];
@@ -638,7 +624,7 @@ delaunay_tri (double *x, double *y, int n, int* nedges)
     struct triangulateio in, out;
     int i;
 
-    in.pointlist = N_GNEW(2 * n, REAL);
+    in.pointlist = gv_calloc(2 * n, sizeof(REAL));
     for (i = 0; i < n; i++) {
 	in.pointlist[2 * i] = x[i];
 	in.pointlist[2 * i + 1] = y[i];
@@ -708,15 +694,13 @@ delaunay_tri (double *x, double *y, int n, int* nedges)
 }
 
 static v_data *delaunay_triangulation(double *x, double *y, int n) {
-    v_data *delaunay;
     int nedges;
-    int *edges;
     int source, dest;
     int* edgelist = delaunay_tri (x, y, n, &nedges);
     int i;
 
-    delaunay = N_GNEW(n, v_data);
-    edges = N_GNEW(2 * nedges + n, int);
+    v_data *delaunay = gv_calloc(n, sizeof(v_data));
+    int *edges = gv_calloc(2 * nedges + n, sizeof(int));
 
     for (i = 0; i < n; i++) {
 	delaunay[i].ewgts = NULL;
@@ -746,42 +730,42 @@ static v_data *delaunay_triangulation(double *x, double *y, int n) {
 surface_t* 
 mkSurface (double *x, double *y, int n, int* segs, int nsegs)
 {
-    agerr (AGERR, "mkSurface not yet implemented using Triangle library\n");
+    agerrorf("mkSurface not yet implemented using Triangle library\n");
     assert (0);
     return 0;
 }
 void 
 freeSurface (surface_t* s)
 {
-    agerr (AGERR, "freeSurface not yet implemented using Triangle library\n");
+    agerrorf("freeSurface not yet implemented using Triangle library\n");
     assert (0);
 }
 #else
 static char* err = "Graphviz built without any triangulation library\n";
 int* get_triangles (double *x, int n, int* tris)
 {
-    agerr(AGERR, "get_triangles: %s\n", err);
+    agerrorf("get_triangles: %s\n", err);
     return 0;
 }
 static v_data *delaunay_triangulation(double *x, double *y, int n) {
-    agerr(AGERR, "delaunay_triangulation: %s\n", err);
+    agerrorf("delaunay_triangulation: %s\n", err);
     return 0;
 }
 int *delaunay_tri(double *x, double *y, int n, int* nedges)
 {
-    agerr(AGERR, "delaunay_tri: %s\n", err);
+    agerrorf("delaunay_tri: %s\n", err);
     return 0;
 }
 surface_t* 
 mkSurface (double *x, double *y, int n, int* segs, int nsegs)
 {
-    agerr(AGERR, "mkSurface: %s\n", err);
+    agerrorf("mkSurface: %s\n", err);
     return 0;
 }
 void 
 freeSurface (surface_t* s)
 {
-    agerr (AGERR, "freeSurface: %s\n", err);
+    agerrorf("freeSurface: %s\n", err);
 }
 #endif
 
@@ -796,16 +780,15 @@ static void remove_edge(v_data * graph, int source, int dest)
     }
 }
 
-v_data *UG_graph(double *x, double *y, int n, int accurate_computation)
-{
+v_data *UG_graph(double *x, double *y, int n) {
     v_data *delaunay;
     int i;
     double dist_ij, dist_ik, dist_jk, x_i, y_i, x_j, y_j;
     int j, k, neighbor_j, neighbor_k;
 
     if (n == 2) {
-	int *edges = N_GNEW(4, int);
-	delaunay = N_GNEW(n, v_data);
+	int *edges = gv_calloc(4, sizeof(int));
+	delaunay = gv_calloc(n, sizeof(v_data));
 	delaunay[0].ewgts = NULL;
 	delaunay[0].edges = edges;
 	delaunay[0].nedges = 2;
@@ -818,8 +801,8 @@ v_data *UG_graph(double *x, double *y, int n, int accurate_computation)
 	delaunay[1].edges[1] = 0;
 	return delaunay;
     } else if (n == 1) {
-	int *edges = N_GNEW(1, int);
-	delaunay = N_GNEW(n, v_data);
+	int *edges = gv_calloc(1, sizeof(int));
+	delaunay = gv_calloc(n, sizeof(v_data));
 	delaunay[0].ewgts = NULL;
 	delaunay[0].edges = edges;
 	delaunay[0].nedges = 1;
@@ -829,70 +812,37 @@ v_data *UG_graph(double *x, double *y, int n, int accurate_computation)
 
     delaunay = delaunay_triangulation(x, y, n);
 
-    if (accurate_computation) {
-	for (i = 0; i < n; i++) {
-	    x_i = x[i];
-	    y_i = y[i];
-	    for (j = 1; j < delaunay[i].nedges;) {
-		neighbor_j = delaunay[i].edges[j];
-		if (neighbor_j < i) {
-		    j++;
-		    continue;
-		}
-		x_j = x[neighbor_j];
-		y_j = y[neighbor_j];
-		dist_ij = (x_j - x_i) * (x_j - x_i) + (y_j - y_i) * (y_j - y_i);
-		bool removed = false;
-		for (k = 0; k < n && !removed; k++) {
-		    dist_ik = (x[k] - x_i) * (x[k] - x_i) + (y[k] - y_i) * (y[k] - y_i);
-		    if (dist_ik < dist_ij) {
-			dist_jk = (x[k] - x_j) * (x[k] - x_j) + (y[k] - y_j) * (y[k] - y_j);
-			if (dist_jk < dist_ij) {
-			    // remove the edge beteween i and neighbor j
-			    delaunay[i].edges[j] = delaunay[i].edges[--delaunay[i].nedges];
-			    remove_edge(delaunay, neighbor_j, i);
-			    removed = true;
-			}
-		    }
-		}
-		if (!removed) {
-		    j++;
-		}
-	    }
-	}
-    } else {
-	// remove all edges v-u if there is w, neighbor of u or v, that is closer to both u and v than dist(u,v)
-	for (i = 0; i < n; i++) {
-	    x_i = x[i];
-	    y_i = y[i];
-	    for (j = 1; j < delaunay[i].nedges;) {
-		neighbor_j = delaunay[i].edges[j];
-		x_j = x[neighbor_j];
-		y_j = y[neighbor_j];
-		dist_ij = (x_j - x_i) * (x_j - x_i) + (y_j - y_i) * (y_j - y_i);
-		// now look at i'th neighbors to see whether there is a node in the "forbidden region"
-		// we will also go through neighbor_j's neighbors when we traverse the edge from its other side
-		bool removed = false;
-		for (k = 1; k < delaunay[i].nedges && !removed; k++) {
-		    neighbor_k = delaunay[i].edges[k];
-		    dist_ik = (x[neighbor_k] - x_i) * (x[neighbor_k] - x_i) +
-			(y[neighbor_k] - y_i) * (y[neighbor_k] - y_i);
-		    if (dist_ik < dist_ij) {
-			dist_jk = (x[neighbor_k] - x_j) * (x[neighbor_k] - x_j) +
-			    (y[neighbor_k] - y_j) * (y[neighbor_k] - y_j);
-			if (dist_jk < dist_ij) {
-			    // remove the edge beteween i and neighbor j
-			    delaunay[i].edges[j] = delaunay[i].edges[--delaunay[i].nedges];
-			    remove_edge(delaunay, neighbor_j, i);
-			    removed = true;
-			}
-		    }
-		}
-		if (!removed) {
-		    j++;
-		}
-	    }
-	}
+    // remove all edges v-u if there is w, neighbor of u or v, that is closer to both u and v than dist(u,v)
+    for (i = 0; i < n; i++) {
+        x_i = x[i];
+        y_i = y[i];
+        for (j = 1; j < delaunay[i].nedges;) {
+            neighbor_j = delaunay[i].edges[j];
+            x_j = x[neighbor_j];
+            y_j = y[neighbor_j];
+            dist_ij = (x_j - x_i) * (x_j - x_i) + (y_j - y_i) * (y_j - y_i);
+            // now look at i'th neighbors to see whether there is a node in the "forbidden region"
+            // we will also go through neighbor_j's neighbors when we traverse the edge from its other side
+            bool removed = false;
+            for (k = 1; k < delaunay[i].nedges && !removed; k++) {
+                neighbor_k = delaunay[i].edges[k];
+                dist_ik = (x[neighbor_k] - x_i) * (x[neighbor_k] - x_i) +
+                    (y[neighbor_k] - y_i) * (y[neighbor_k] - y_i);
+                if (dist_ik < dist_ij) {
+                    dist_jk = (x[neighbor_k] - x_j) * (x[neighbor_k] - x_j) +
+                        (y[neighbor_k] - y_j) * (y[neighbor_k] - y_j);
+                    if (dist_jk < dist_ij) {
+                        // remove the edge beteween i and neighbor j
+                        delaunay[i].edges[j] = delaunay[i].edges[--delaunay[i].nedges];
+                        remove_edge(delaunay, neighbor_j, i);
+                        removed = true;
+                    }
+                }
+            }
+            if (!removed) {
+                j++;
+            }
+        }
     }
     return delaunay;
 }
