@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -55,3 +55,7 @@
 
 void find_ints(struct vertex vertex_list[], struct data *input,
                struct intersection ilist[]);
+
+/// detect whether lines `l` and `m` intersect
+void find_intersection(struct vertex *l, struct vertex *m,
+                       struct intersection ilist[], struct data *input);
